# Generated by Django 3.1.1 on 2020-10-11 03:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_oci", "0004_auto_20200920_2322"),
    ]

    operations = [
        migrations.AlterField(
            model_name="image",
            name="manifest",
            field=models.BinaryField(default=b"{}"),
        ),
    ]
