# Generated by Django 3.1.1 on 2020-09-20 22:37

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("django_oci", "0002_auto_20200920_0155"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="annotation",
            name="images",
        ),
        migrations.AddField(
            model_name="annotation",
            name="image",
            field=models.ForeignKey(
                default=None,
                on_delete=django.db.models.deletion.CASCADE,
                to="django_oci.image",
            ),
            preserve_default=False,
        ),
    ]
