# Generated by Django 3.1.1 on 2020-09-20 01:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("django_oci", "0001_initial"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="blob",
            unique_together={("repository", "digest")},
        ),
    ]
