# Generated by Django 3.1.1 on 2020-10-12 04:38

from django.db import migrations, models
import django_oci.models


class Migration(migrations.Migration):

    dependencies = [
        ("django_oci", "0005_auto_20201011_0325"),
    ]

    operations = [
        migrations.AlterField(
            model_name="blob",
            name="datafile",
            field=models.FileField(
                max_length=255,
                storage=django_oci.models.OverwriteStorage(),
                upload_to=django_oci.models.get_upload_folder,
            ),
        ),
    ]
